<?php
/**
 * @package XML_RPC5
 * @author Greg Beaver <cellog@php.net>
 */
/**
 * Utility class
 */
require_once 'Chiara/XML/RPC5.php';
/**
 * XML_RPC client.
 *
 * To use is straightforward.
 *
 * <code>
 * require_once 'Chiara/XML/RCP5/Client.php';
 * $xmlrpc = new Chiara_XML_RPC5_Client('http://pear.chiaraquartet.net/xmlrpc.php');
 * $ret = $xmlrpc->{"package.info"}('PEAR_Server'); // this
 * $ret = $xmlrpc->call('package.info', 'PEAR_Server'); // or this
 *
 * class MyRpc extends Chiara_XML_RPC5_Client
 * {
 *     protected $rpcclass = 'package';
 * }
 * $xmlrpc = new Chiara_XML_RPC5_Client('http://pear.chiaraquartet.net/xmlrpc.php');
 * $ret = $xmlrpc->info('PEAR_Server');
 * ?>
 *
 * Caching is supported through the use of an external callback, and a proxy server is
 * supported as an optional 2nd parameter to the constructor
 * @package Chiara_XML_RPC5
 * @author Greg Beaver <cellog@php.net>
 */
class Chiara_XML_RPC5_Client
{
    private $_protocol;
    private $_proxy;
    protected $rpcclass;
    protected $cacheCallback = false;

    public function __construct($url, $proxy = false)
    {
        $op = $proxy;
        if ($proxy) {
            if ($proxy = parse_url($proxy)) {
                if (!$proxy || !isset($proxy['host']) || !isset($proxy['scheme'])) {
                    throw new Chiara_XML_RPC5_Exception('Invalid proxy in constructor: ' . $op);
                }
                $this->_proxy = $proxy;
                foreach (array('host', 'port', 'user', 'pass', 'scheme') as $part) {
                    if (!isset($this->_proxy[$part])) {
                        $this->_proxy[$part] = false;
                    } else {
                        $this->_proxy[$part] = urldecode($this->_proxy[$part]);
                    }
                }
            }
        }
        $this->_protocol = parse_url($url);
        if (!$this->_protocol || !isset($this->_protocol['host']) ||
              !isset($this->_protocol['scheme']) || !isset($this->_protocol['path'])) {
            throw new Chiara_XML_RPC5_Exception('Invalid URL in constructor: ' . $url);
        }
        foreach (array('scheme', 'host', 'user', 'pass', 'path', 'port', 'query') as $part) {
            if (!isset($this->_protocol[$part])) {
                $this->_protocol[$part] = false;
            } else {
                $this->_protocol[$part] = urldecode($this->_protocol[$part]);
            }
        }
        if (!$this->_protocol['port']) {
            $this->_protocol['port'] = 80;
        }
    }

    public function __call($name, $args)
    {
        if (get_class($this) != 'Chiara_XML_RPC5_Client') {
            if (isset($this->rpcclass)) {
                $name = $this->rpcclass . '.' . $name;
            } else {
                $name = str_replace('Chiara_XML_RPC5_Client_', '', get_class($this))
                    . '.' . $name;
            }
        }
        array_unshift($args, $name);
        return call_user_func_array(array($this, 'call'), $args);
    }

    public function call($name)
    {
        $args = func_get_args();
        array_shift($args);
        $request = Chiara_XML_RPC5::factory()->encodeRequest($name, $args);
        if (isset($this->_proxy) && $this->_proxy['host']) {
            $host = $this->_proxy['host'];
            if ($this->_proxy['scheme']) {
                if ($this->_proxy['scheme'] == 'https') {
                    $host = 'ssl://' . $host;
                }
            }
            if (!$this->_proxy['port']) {
                $port = 80;
            } else {
                $port = $this->_proxy['port'];
            }
            
            $fp = @fsockopen($host, $port, $errno, $errstr, 20);
            if (!$fp) {
                throw new Chiara_XML_RPC5_CallException("fsockopen($host, $port) failed: $errstr");
            }
        } else {
            $port = ($this->_protocol['port']) ? $this->_protocol['port'] : 80;
            $fp = @fsockopen($this->_protocol['host'], $port, $errno, $errstr, 20);
            if (!$fp) {
                throw new Chiara_XML_RPC5_CallException("fsockopen($host, $port) failed: $errstr");
            }
        }
        $len = strlen($request);
        $req_headers = 'Host: ' . $this->_protocol['host'] . ':' . $this->_protocol['port'] .
             "\r\n" .
             "Content-type: text/xml\r\n" .
             "Content-length: $len\r\n";
        if ($this->_proxy && $this->_proxy['user'] && $this->_proxy['pass']) {
            $req_headers .= 'Proxy-Authorization: Basic '
                . base64_encode($this->_proxy['user'] . ':' . $this->_proxy['pass'])
                . "\r\n";
        }
        if ($this->_proxy) {
            $post = "POST http://" . $this->_protocol['host'];
            if ($this->_protocol['port']) {
                $post .= ':' . $this->_protocol['port'];
            }
        } else {
            $post = 'POST ';
        }
        $path = $this->_protocol['path'];
        if ($this->_protocol['query']) {
            $path .= '?' . $this->_protocol['query'];
        }
        fwrite($fp, ($post . $path . " HTTP/1.0\r\n" .
            "$req_headers\r\n$request"));
        $response = '';
        $line1 = fgets($fp, 2048);
        if (!preg_match('!^HTTP/[0-9\.]+ (\d+) (.*)!', $line1, $matches)) {
            throw new Chiara_XML_RPC5_CallException('invalid HTTP response from xml-rpc server');
        }
        switch ($matches[1]) {
            case "200": // OK
                break;
            case "304": // Not Modified
                if ($this->getCache($method, $params)) {
                    return;
                }
            case "401": // Unauthorized
                if ($username && $password) {
                    throw new Chiara_XML_RPC5_CallException('authorization failed', 401);
                } else {
                    throw new Chiara_XML_RPC5_CallException('authorization required, please log in first', 401);
                }
            default:
                throw new Chiara_XML_RPC5_CallException('unexpected HTTP response. Code: ' . $matches[1] . ', extra: ' .$matches[2]);
        }
        while (trim(fgets($fp, 2048)) != ''); // skip rest of headers
        while ($chunk = fread($fp, 10240)) {
            $response .= $chunk;
        }
        fclose($fp);
        return Chiara_XML_RPC5::factory()->decodeResponse($response);
    }

    function setCacheCallback($callback)
    {
        if (is_callable($callback)) {
            $this->cacheCallback = $callback;
        } else {
            $this->cacheCallback = false;
        }
    }

    function getCache($method, $params)
    {
        if ($this->cacheCallback) {
            return call_user_func_array($this->cacheCallback, array($method, $params));
        }
    }
}
?>