<?php
/**
 * XML_RPC5 Server
 * @package XML_RPC5
 * @author Greg Beaver <cellog@php.net>
 */
require_once 'Chiara/XML/RPC5.php';
require_once 'Chiara/XML/RPC5/Interface.php';
class Chiara_XML_RPC5_ServerException extends Chiara_XML_RPC5_Exception {}
/**
 * Server engine for PHP5
 *
 * The server class is designed to be instance-based.  You must either
 * extend the server class, or implement the Chiara_XML_RPC5_Interface_Server.
 *
 * You can create a server for a single classname/uri combination, with the
 * option of handling all classes in a single server.
 *
 * The use of the server is through the static run() function.
 *
 * <code>
 * <?php
 * class MyServer extends Chiara_XML_RPC5_Server
 * {
 *     /**
 *      * @param string
 *      * @return string
 *      {@*}
 *     public function test($a)
 *     {
 *         return "Hello, $a";
 *     }
 * }
 * $server = new MyServer('/xmlrpc.php', 'test');
 * ?>
 * </code>
 *
 * Using the documentation in standard format above, the server will automatically
 * register a signature of string Myserver(string), and validate all input against this
 * signature.  No input validation is necessary in the server class, you can write just
 * as you normally would.
 *
 * A @noxmlrpc tag can be used to explicitly ignore a public function.  All protected and
 * private functions are ignored.  This package required PHP 5.1 to fully process
 * all optional parameters due to new features in Reflection.
 * @package Chiara_XML_RPC5
 * @author Greg Beaver <cellog@php.net>
 */
abstract class Chiara_XML_RPC5_Server
{
    private $_server;
    private $_input;
    private $_uri;
    private static $_servers = array();
    public static $methodMap = array();

    public function __construct($classname = '*', $introspect = 'introspect')
    {
        if (isset(self::$_servers[$classname])) {
            throw new Chiara_XML_RPC5_ServerException('Cannot create multiple servers for URI "' .
                $uri . '" and class "' . $classname . '"');
        }
        self::$_servers[$classname] = $this;
    }

    public static function registerServer(Chiara_XML_RPC5_Interface_Server $server)
    {
        if (isset(self::$_servers[$server->getXmlrpcServerClassName()])) {
            throw new Chiara_XML_RPC5_ServerException('Cannot create multiple servers for URI "' .
                $uri . '" and class "' . $classname . '"');
        }
    }

    public function introspect($method, $params)
    {
        foreach (self::$_servers as $class => $obj) {
            $signatures = array_merge($signatures, self::getSignatures($class, $obj));
        }
        if ($method == 'system.listMethods') {
            return array_keys($signatures);
        }
        if ($method == 'system.methodSignature') {
            if (isset($signatures[$params[0]])) {
                $ret = array();
                foreach ($signatures[$params[0]] as $sig) {
                    $arr = $sig['params'];
                    array_unshift($arr, $sig['returns']);
                    $ret[] = $arr;
                }
                if ($ret == array()) {
                    return null;
                }
                return $ret;
            }
            return null;
        }
        if ($method == 'system.methodHelp') {
            if (isset($signatures[$params[0]])) {
                return $signatures[$params[0]]['summary'];
            }
        }
    }

    static public function run()
    {
        if (isset($GLOBALS['HTTP_RAW_POST_DATA'])) {
            $input = $GLOBALS['HTTP_RAW_POST_DATA'];
        } else {
            $input = file_get_contents('php://input');
        }
        try {
            $params = Chiara_XML_RPC5::factory()->decodeRequest($input, $method);
        } catch (PEAR_Exception $e) {
            throw new Chiara_XML_RPC5_ServerException('invalid input xml', $e);
        }
        $usemethod = $method;
        if (strpos($method, '.')) {
            $class = array_shift(explode('.', $method));
            $method = array_pop(explode('.', $method));
        } else {
            $class = 'function';
        }
        if (!isset(self::$_servers[$class])) {
            if (isset(self::$_servers['*'])) {
                $method = $class . '_' . $method;
                $class = '*';
            } else {
                if ($class == 'system') {
                    return self::introspect($method, $params);
                }
                throw new Chiara_XML_RPC5_ServerException('No class handler for "' . $class . '"' .
                    ' or generic XML-RPC handler has been registered');
            }
        }
        if (self::$_servers[$class]
              instanceof Chiara_XML_RPC5_Interface_Server) {
            $a = self::$_servers[$class]->getXmlrpcServerMethodMap();
        } else {
            $a = new ReflectionProperty(
                get_class(self::$_servers[$class]),
                'methodMap');
            $a = $a->getValue(self::$_servers[$class]);
        }
        if (is_array($a) && isset($a[$method])) {
            $method = $a[$method];
        }
        if (self::validSignature(self::$_servers[$class],
              $usemethod, $params, $class)) {
            if (method_exists(self::$_servers[$class],
                  $method)) {
                $ret = call_user_func_array(
                    array(self::$_servers[$class], $method),
                $params);
                $ret = Chiara_XML_RPC5::factory()->encodeResponse($ret);
                header("Content-length: " . strlen($ret));
                header("Content-type: text/xml");
                print $ret;
            } else {
                throw new Chiara_XML_RPC5_ServerException('Unknown method: ' . $method, -32601);
            }
        } else {
            throw new Chiara_XML_RPC5_ServerException('Invalid signature "' . self::_signatureToString(
                $params)
                . '" for method "' . $usemethod . '"');
        }
    }

    static private function _signatureToString($sig)
    {
        $ret = '';
        foreach ($sig as $item) {
            if (!empty($ret)) {
                $ret .= ', ';
            }
            $ret .= gettype($item);
            if (is_object($item)) {
                $ret .= ' ' . get_class($item);
            }    
        }
        return $ret;
    }

    static public function getSignatures($rpcclass, $phpclass)
    {
        if (is_object($phpclass)) {
            if ($phpclass instanceof Chiara_XML_RPC5_Interface_Server) {
                return $phpclass->getSignatures();
            }
            $phpclass = get_class($phpclass);
        }
        $c = new ReflectionClass($phpclass);
        foreach ($c->getMethods() as $method) {
            if (!$method->isPublic() || $method->isAbstract() || $method->isConstructor()) {
                continue;
            }
            if ($method->getName() == 'introspect' || $method->getName() == '__destruct' ||
                  $method->getName() == 'run' || $method->getName() == 'call' ||
                  $method->getName() == 'getSignatures' ||
                  $method->getName() == 'registerServer' ||
                  $method->getName() == 'validSignature') {
                continue;
            }
            if ($c->implementsInterface('Chiara_XML_RPC5_Interface_Server')) {
                if ($method->getName() == 'getXmlrpcServerMethodMap' ||
                      $method->getName() == 'getXmlrpcServerUri' ||
                      $method->getName() == 'getXmlrpcServerClassName') {
                    continue;
                }
            }
            $methodParams = self::_parseDocs($method);
            if ($methodParams === false) { // @noxmlrpc tag found
                continue;
            }
            $name = $method->getName();
            if ($rpcclass == '*') {
                $name = str_replace('_', '.', $name);
            } elseif ($rpcclass != 'function') {
                $name = $rpcclass . '.' . $name;
            }
            $signatures[$name] = array(
                'params' => $methodParams['params'],
                'returns' => $methodParams['returns']);
        }
        $signatures['system.listMethods'] =
            array('params' => array(), 'returns' => 'array', 'summary' => 'list all XML-RPC methods');
        $signatures['system.methodSignature'] =
            array('params' => array(array('type' => 'string')), 'returns' => 'array',
                'summary' => 'get all signatures for a method (return value is first)');
        $signatures['system.methodHelp'] =
            array('params' => array(array('type' => 'string')), 'returns' => 'string',
                'summary' => 'get the summary of a method\'s purpose');
        return $signatures;
    }

    static public function validSignature($obj, $method, $params, $class)
    {
        $sigs = self::getSignatures($class, $obj);
        if ($sigs === false) { // @noxmlrpc tag found
            return false;
        }
        if (!isset($sigs[$method])) {
            throw new Chiara_XML_RPC5_ServerException('Unknown method: ' . $method, -32601);
        }
        foreach ($sigs[$method]['params'] as $i => $param) {
            if ($param['optional'] && (!isset($params[$i]) || is_null($params[$i]))) {
                continue;
            }
            if (is_array($param['type'])) {
                if (in_array(gettype($params[$i]), $param['type'])) {
                    continue;
                }
                if (gettype($params[$i]) == 'object') {
                    if (in_array(get_class($params[$i]), $param['type'])) {
                        continue;
                    }
                    return false;
                }
            } else {
                if (!array_key_exists($i, $params)) {
                    return false;
                }
                if (gettype($params[$i]) == $param['type']) {
                    continue;
                }
                if (gettype($params[$i]) == 'object') {
                    if (get_class($params[$i]) == $param['type']) {
                        continue;
                    }
                    return false;
                }
            }
            return false;
        }
        return true;
    }

    static private function _parseDocs($method)
    {
        $docs = $method->getDocComment();
        $params = array();
        foreach ($method->getParameters() as $param) {
            $par = array('optional' => false);
            if (method_exists($method, 'isOptional')) {
                $par['optional'] = $param->isOptional();
            }
            if ($param->getClass()) {
                $par['type'] = $param->getClass();
            }
            $params[] = $par;
        }
        $returns = 'mixed';
        $shortdesc = '';
        if (!$docs) {
            return false; // don't expose undocumented functions
        }
        $docs = explode("\n", $docs);
        $paramcount = -1;
        foreach ($docs as $i => $doc) {
            $doc = trim($doc, " \r\t/*");
            if (strlen($doc) && strpos($doc, '@') !== 0) {
                if ($shortdesc) {
                    $shortdesc .= "\n";
                }
                $shortdesc .= $doc;
                continue;
            }
            if (strpos($doc, '@noxmlrpc') === 0) {
                return false;
            }
            if (strpos($doc, '@param') === 0) {
                $paramcount++;
                $param = preg_split("/\s+/", $doc);
                if (isset($param[1])) {
                    $param = $param[1];
                    if (strpos($param, '|')) {
                        $param = explode('|', $param);
                    }
                }
                if (isset($params[$paramcount])) {
                    if (!isset($params[$paramcount]['type'])) {
                        $params[$paramcount]['type'] = $param;
                    }
                } else {
                    $params[$paramcount] = array('type' => $param, 'optional' => true);
                }
            }
            if (strpos($doc, '@return') === 0) {
                $param = preg_split("/\s+/", $doc);
                if (isset($param[1])) {
                    $param = $param[1];
                    $returns = $param;
                }
            }
        }
        return array('params' => $params, 'returns' => $returns, 'summary' => $shortdesc);
    }

    static private function fault($e)
    {
        $fault = Chiara_XML_RPC5::factory()->encodeResponse(array('faultString' => $e->getMessage(), 'faultCode' => $e->getCode()));
        header("Content-length: " . strlen($fault));
        header("Content-type: text/xml");
        print $fault;
        exit;
    }
}
?>