<?php
/**
 * XML_RPC driver class for non-xmlrpc systems in PHP5
 * @package Chiara_XML_RPC5
 * @author Greg Beaver <cellog@php.net>
 */
class Chiara_XML_RPC5_php5 extends Chiara_XML_RPC5
{
    public function encode($val)
    {
        Chiara_XML_RPC5_Custom::$asXml = true;
        if ($this->_request) {
            $ret = '';
        } else {
            $ret = "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n";
        }
        if ($val === null) {
            $ret .= "<params/>\n";
        } else {
            $ret .= "<params>\n";
            $ret .= "<param>\n";
            $ret .= " <value>\n";
            $ret .= $this->_parseVal($val, '  ');
            $ret .= " </value>\n";
            $ret .= "</param>\n";
            $ret .= "</params>\n";
        }
        Chiara_XML_RPC5_Custom::$asXml = false;
        return $ret;
    }

    private function _parseVal($val, $indent = '')
    {
        if (is_scalar($val)) {
            switch (gettype($val)) {
                case 'boolean' :
                    return "$indent<boolean>" . (int) $val . "</boolean>\n";
                case 'integer' :
                    return "$indent<int>$val</int>\n";
                case 'double' :
                    return "$indent<double>$val</double>\n";
                case 'string' :
                    return "$indent<string>" . htmlspecialchars($val) . "</string>\n";
            }
        }
        if (is_null($val)) {
            return "$indent<string/>\n";
        }
        if (is_array($val)) {
            $keys = array_keys($val);
            for($i = 0; $i < count($keys); $i++) {
                if ($keys[$i] !== $i) {
                    return $this->_parseStruct($val, "$indent");
                }
            }
            return $this->_parseArray($val, $indent);
        }
        if (is_object($val)) {
            if ($val instanceof Chiara_XML_RPC5_Custom) {
                return $indent . $val->__toString() . "\n";
            } else {
                return $this->_parseStruct($val, $indent);
            }
        }
    }

    private function _parseArray($val, $indent = '')
    {
        $ret = "$indent<array>\n";
        if (count($val)) {
            $ret .= "$indent <data>\n";
            foreach ($val as $subval) {
                $ret .= "$indent  <value>\n";
                $ret .= $this->_parseVal($subval, "$indent   ");
                $ret .= "$indent  </value>\n";
            }
            $ret .= "$indent </data>\n";
        } else {
            $ret .= "$indent <data/>\n";
        }
        return "$ret$indent</array>\n";
    }

    private function _parseStruct($val, $indent = '')
    {
        $ret = "$indent<struct>\n";
        if (is_object($val)) {
            $iter = new ArrayObject($val);
        } else {
            $iter = $val;
        }
        if (count($val)) {
            foreach ($iter as $name => $subval) {
                $ret .= "$indent <member>\n";
                if (is_int($name)) {
                    $ret .= "$indent  <name/>\n";
                } else {
                    $ret .= "$indent  <name>$name</name>\n";
                }
                $ret .= "$indent  <value>\n";
                $ret .= $this->_parseVal($subval, "$indent   ");
                $ret .= "$indent  </value>\n";
                $ret .= "$indent </member>\n";
            }
        }
        return "$ret$indent</struct>\n";
    }

    public function encodeResponse($val)
    {
        return $this->encodeRequest(null, $val);
    }

    public function encodeRequest($method, $val)
    {
        $this->_request = true;
        try {
            $ret = "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n";
            if ($method === null) {
                $ret .= "<methodResponse>\n";
            } else {
                $ret .= "<methodCall>\n";
                $ret .= "<methodName>$method</methodName>\n";
            }
            $ret .= $this->encode($val);
            if ($method === null) {
                $ret .= "</methodResponse>\n";
            } else {
                $ret .= "</methodCall>\n";
            }
            $this->_request = false;
            return $ret;
        } catch (Exception $e) {
            $this->_request = false;
            throw $e;
        }
    }

    public function decode($val, $encoding = null)
    {
        if (is_string($val)) {
            $val = simplexml_load_string($val);
            if (!$val || !$this->validParams($sxe)) {
                throw new Chiara_XML_RPC5_Exception('Invalid xml input');
            }
        }
        $ret = array();
        foreach ($val->param as $param) {
            $ret[] = $this->_decodeParam($param);
        }
        return $ret;
    }

    private function _decodeParam($param)
    {
        if (!isset($param->value)) {
            return null;
        }
        foreach ($param->value->children() as $name => $node) {
            return $this->_decodeVal($name, $node);
        }
    }

    private function _decodeVal($name, $node)
    {

        switch ($name) {
            case 'string' :
                return (string) $node;
            case 'boolean' :
                return (boolean) ((string) $node);
            case 'i4' :
            case 'int' :
                return (int) ((string) $node);
            case 'double' :
                return (double) ((string) $node);
            case 'array' :
                return $this->_decodeArray($node);
            case 'struct' :
                return $this->_decodeStruct($node);
            case 'dateTime.iso8601' :
                return strtotime((string) $node);
            case 'base64' :
                return base64decode((string) $node);
        }
    }

    private function _decodeArray($node)
    {
        $ret = array();
        if (isset($node->data->value)) {
            foreach ($node->data->value as $value) {
                foreach ($value->children() as $name => $subnode) {
                    $ret[] = $this->_decodeVal($name, $subnode);
                }
            }
        }
        return $ret;
    }

    private function _decodeStruct($node)
    {
        $ret = array();
        if (isset($node->member)) {
            foreach ($node->member as $member) {
                foreach ($member->value->children() as $name => $subnode) {
                    if ((string) $member->name == '') {
                        $ret[] = $this->_decodeVal($name, $subnode);
                    } else {
                        $ret[(string) $member->name] = $this->_decodeVal($name, $subnode);
                    }
                }
            }
        }
        return $ret;
    }

    public function decodeRequest($val, &$method, $encoding = null)
    {
        $sxe = simplexml_load_string($val);
        if (!$sxe || !$this->validRequest($sxe, $method)) {
            throw new Chiara_XML_RPC5_Exception('Invalid xml input');
        }
        if (isset($sxe->fault)) {
            throw new Chiara_XML_RPC5_FaultException(
                $sxe->fault->value->struct->member[1]->value->string,
                $sxe->fault->value->struct->member[0]->value->int);
        } else {
            if (isset($sxe->methodName)) {
                $method = (string) $sxe->methodName;
            }
            if (isset($sxe->params->param->value->array)) {
                return $this->decode($sxe->params);
            }
            return $this->decode($sxe->params);
        }
    }

    public function decodeResponse($val, $encoding = null)
    {
        $null = null;
        $this->decodeRequest($val, $null, $encoding);
    }

    public function getType($val)
    {
        if ($val instanceof Chiara_XML_RPC5_Custom) {
            return $val->getType();
        }
        if (is_scalar($val)) {
            if (is_int($val)) {
                return 'int';
            }
            return gettype($val);
        }
        if (is_null($val)) {
            return 'string';
        }
        if (is_array($val)) {
            $keys = array_keys($val);
            for($i = 0; $i < count($keys); $i++) {
                if ($keys[$i] !== $i) {
                    return 'struct';
                }
            }
            return 'array';
        }
        if (is_object($val)) {
            return 'struct';
        }
    }

    public function setType(&$val, $type)
    {
        parent::setType($val, $type);
        try {
            $val = new Chiara_XML_RPC5_Custom($val, $type);
            return true;
        } catch (Exception $e) {
            return false;
        }
    }
}

/**
 * Private class, used to represent dateTime and base64 types
 * @package XML_RPC5
 */
class Chiara_XML_RPC5_Custom
{
    private $_type;
    private $_value;
    
    public static $asXml = true;
    public function __construct($value, $type)
    {
        if ($type == 'datetime') {
            $type = 'dateTime.iso8601';
            $this->_value = strtotime($value);
            if ($this->_value == -1) {
                $type = 'string';
            } else {
                $value = strftime('%Y%m%dT%H:%M:%S', $this->_value);
            }
        }
        $this->_type = $type;
        $this->_value = $value;
        if ($type == 'string') {
            throw new Exception('dummy');
        }
    }

    public function getType()
    {
        return $this->_type;
    }

    public function __toString()
    {
        if (self::$asXml) {
            return $this->_asXml();
        } else {
            return $this->_value;
        }
    }

    private function _asXml()
    {
        $ret = '<' . $this->_type . '>';
        if ($this->_type == 'base64') {
            $ret .= base64_encode($this->_value);
        } else {
            $ret .= $this->_value;
        }
        $ret .= '</' . $this->_type . '>';
        return $ret;
    }
}
?>